/**
 * PETALS - PETALS Services Platform. Copyright (c) 2008 EBM Websourcing,
 * http://www.ebmwebsourcing.com/
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version. This library is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 */
package org.ow2.petals.communication.jmx;

import javax.management.MBeanServer;

import org.objectweb.fractal.api.control.IllegalLifeCycleException;
import org.objectweb.fractal.fraclet.annotation.annotations.FractalComponent;
import org.objectweb.fractal.fraclet.annotation.annotations.Interface;
import org.objectweb.fractal.fraclet.annotation.annotations.LifeCycle;
import org.objectweb.fractal.fraclet.annotation.annotations.Monolog;
import org.objectweb.fractal.fraclet.annotation.annotations.Provides;
import org.objectweb.fractal.fraclet.annotation.annotations.Requires;
import org.objectweb.fractal.fraclet.annotation.annotations.type.LifeCycleType;
import org.objectweb.fractal.jmx.agent.AdminAttributes;
import org.objectweb.util.monolog.api.Logger;
import org.ow2.petals.kernel.api.server.PetalsException;
import org.ow2.petals.util.LoggingUtil;

/**
 * The Standalone implementation of the JMX Service.
 * 
 * @author rnaudin - EBM Websourcing
 * @since PEtALS 2.3
 */
@FractalComponent
@Provides(interfaces = @Interface(name = "service", signature = org.ow2.petals.communication.jmx.JMXService.class))
public class StandaloneJMXServiceImpl implements JMXService {

    /**
     * Logger
     */
    protected LoggingUtil log;

    @Monolog(name = "logger")
    private Logger logger;

    /**
     * AdminAttributes
     */
    @Requires(name = "adminAtt", signature = org.objectweb.fractal.jmx.agent.AdminAttributes.class)
    protected AdminAttributes adminAttributes;

    private RemoteJMXServer distributedJMXServer;

    /*
     * (non-Javadoc)
     * 
     * @see
     * org.ow2.petals.communication.jmx.JMXService#createDistributedJMXServer
     * (java.lang.String)
     */
    public RemoteJMXServer getRemoteJMXServer(String containerName)
            throws PetalsException {
        this.log.call(containerName);

        if (distributedJMXServer == null) {
            distributedJMXServer = new RemoteJMXServer(this.getLocalJMXServer());
        }
        return distributedJMXServer;
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.ow2.petals.communication.jmx.JMXService#getLocalJMXServer()
     */
    public MBeanServer getLocalJMXServer() {
        this.log.call();

        return this.adminAttributes.getRawMBeanServer();
    }

    @LifeCycle(on = LifeCycleType.START)
    protected void start() throws IllegalLifeCycleException {
        this.log = new LoggingUtil(this.logger);
        this.log.call();
    }

    @LifeCycle(on = LifeCycleType.STOP)
    protected void stop() throws IllegalLifeCycleException {
        this.log.call();
    }

}
