/**
 * PETALS: PETALS Services Platform Copyright (C) 2005 EBM WebSourcing
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA.
 * 
 * Initial developer(s): EBM WebSourcing
 * --------------------------------------------------------------------------
 * $Id: MBeanNamesImpl.java,v 1.2 2005/07/22 10:24:27 alouis Exp $
 * --------------------------------------------------------------------------
 */

package org.ow2.petals.jbi.management;

import javax.jbi.management.MBeanNames;
import javax.management.MalformedObjectNameException;
import javax.management.ObjectName;

/**
 * Implement the JBI <code>MBeanNames</code> element, which is offered to
 * components for creating mBean names for their optional mBean elements. It
 * also offers methods to the <code>Manager</code> for creating JBI elements
 * mBean names (binding and engine components, and system services components).
 * 
 * @author Adrien LOUIS - EBM WebSourcing
 */
public class MBeanNamesImpl implements MBeanNames {

    private static final String BINDING = "binding";

    private static final String CUSTOM = "custom";

    private static final String ENGINE = "engine";

    private static final String INSTALLER = "installer";

    private static final String SYSTEM = "system";

    private String domain;

    /**
     * Constructs the MBeanNames with the specified domain name. This domain
     * name will be used for creating mBean names.
     * 
     * @param domain
     */
    public MBeanNamesImpl(String domain) {
        this.domain = domain;
    }

    /**
     * 
     * @param name
     * @return
     */
    public ObjectName createBindingComponentMBeanName(String name) {
        return createMBeanName(name, BINDING);
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * javax.jbi.management.MBeanNames#createCustomComponentMBeanName(java.lang
     * .String)
     */
    public ObjectName createCustomComponentMBeanName(String customName) {
        return createMBeanName(customName, CUSTOM);
    }

    /**
     * 
     * @param name
     * @return
     */
    public ObjectName createEngineComponentMBeanName(String name) {
        return createMBeanName(name, ENGINE);
    }

    /**
     * 
     * @param name
     * @return
     */
    public ObjectName createInstallerMBeanName(String name) {
        return createMBeanName(name, INSTALLER);
    }

    /**
     * Create a MBean name
     * 
     * @param name
     * @param type
     * @return
     */
    protected ObjectName createMBeanName(String name, String type) {
        ObjectName on = null;
        String theType = type != null ? "type=" + type + "," : "";
        String theName = "name=" + name;

        try {
            on = new ObjectName(domain + ":" + theType + theName);
        } catch (MalformedObjectNameException e) {
            throw new RuntimeException(e);
            // internal use, so exeception should never be thrown
        }
        return on;
    }

    /**
     * 
     * @param name
     * @return
     */
    public ObjectName createSystemComponentMBeanName(String name) {
        return createMBeanName(name, SYSTEM);
    }

    /**
     * 
     */
    public String getJmxDomainName() {
        return domain;
    }

    /**
     * 
     * @param objectName
     * @return
     */
    public String getNameFromMBeanName(ObjectName objectName) {
        return objectName.getKeyProperty("name");
    }
}
