/*
 * Copyright (c) 2009 EBM Websourcing, http://www.ebmwebsourcing.com/
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version. This library is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */
package org.ow2.petals.jbi.management.deployment;

import java.net.URL;

import org.objectweb.fractal.fraclet.annotation.annotations.FractalComponent;
import org.objectweb.fractal.fraclet.annotation.annotations.Interface;
import org.objectweb.fractal.fraclet.annotation.annotations.LifeCycle;
import org.objectweb.fractal.fraclet.annotation.annotations.Monolog;
import org.objectweb.fractal.fraclet.annotation.annotations.Provides;
import org.objectweb.fractal.fraclet.annotation.annotations.Requires;
import org.objectweb.fractal.fraclet.annotation.annotations.type.LifeCycleType;
import org.objectweb.util.monolog.api.Logger;
import org.ow2.petals.kernel.api.server.PetalsException;
import org.ow2.petals.util.LoggingUtil;

/**
 * This simple deployment service hides all the JMX stuff and the reports
 * handling. When something is OK it returns 'true', if not then 'false' (Waow,
 * this is powerful!!! :D ). It is deicated to be used with only a SA at a time,
 * no more lists...
 * 
 * @author Christophe HAMERLING - eBM WebSourcing
 * 
 */
@FractalComponent
@Provides(interfaces = @Interface(name = "service", signature = org.ow2.petals.jbi.management.deployment.AtomicDeploymentService.class))
public class AtomicDeploymentServiceImpl implements AtomicDeploymentService {

    @Requires(name = "deployment", signature = DeploymentServiceMBean.class)
    private DeploymentServiceMBean deploymentService;

    @Monolog(name = "logger")
    private Logger logger;

    private LoggingUtil log;

    @LifeCycle(on = LifeCycleType.START)
    public void start() {
        this.log = new LoggingUtil(logger);
    }

    @LifeCycle(on = LifeCycleType.STOP)
    public void stop() {

    }

    public boolean deploy(URL saURL) throws PetalsException {
        boolean result = true;
        if (saURL == null) {
            throw new PetalsException("Service assembly URL is null");
        }

        try {
            String report = this.deploymentService.deploy(saURL.toString());
            // really really bad...
            result = (report.indexOf("Successfully") > -1);

        } catch (Exception e) {
            log.warning(e.getMessage());
            throw new PetalsException(e.getMessage());
        }
        return result;
    }

    public boolean shutdown(String saId) throws PetalsException {
        boolean result = true;
        try {
            String report = this.deploymentService.shutDown(saId);
            // really really bad...
            result = (report.indexOf("Successfully") > -1);

        } catch (Exception e) {
            log.warning(e.getMessage());
            throw new PetalsException(e.getMessage());
        }
        return result;
    }

    public boolean start(String saId) throws PetalsException {
        boolean result = true;
        try {
            String report = this.deploymentService.start(saId);
            // really really bad...
            result = (report.indexOf("Successfully") > -1);

        } catch (Exception e) {
            log.warning(e.getMessage());
            throw new PetalsException(e.getMessage());
        }
        return result;
    }

    public boolean stop(String saId) throws PetalsException {
        boolean result = true;
        try {
            String report = this.deploymentService.stop(saId);
            // really really bad...
            result = (report.indexOf("Successfully") > -1);

        } catch (Exception e) {
            log.warning(e.getMessage());
            throw new PetalsException(e.getMessage());
        }
        return result;
    }

    public boolean undeploy(String saId) throws PetalsException {
        boolean result = true;
        try {
            String report = this.deploymentService.undeploy(saId);
            // really really bad...
            result = (report.indexOf("Successfully") > -1);

        } catch (Exception e) {
            log.warning(e.getMessage());
            throw new PetalsException(e.getMessage());
        }
        return result;
    }

}
