/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: InstallationServiceMBean.java 154 2006-03-27 15:30:10Z ofabre $
 * -------------------------------------------------------------------------
 */
package org.ow2.petals.jbi.management.deployment;


/**
 * This custom interface add methods used by redeployment tasks of the
 * SystemStateService service.
 * 
 * @author ofabre
 * 
 */
public interface DeploymentServiceMBean extends
    javax.jbi.management.DeploymentServiceMBean {

    /**
     * Try to undeploy a service assembly. If the SA is in the STARTED state, it
     * will be previously stopped and shut down. If the SA is in the STOPPED state, it will
     * be previously shut down.
     * 
     * @return true if the SA is undeployed, false otherwise.
     */
    boolean forceUndeploy(String name);

    /**
     * Get all the service unit names for the given service assembly
     * 
     * @param saName
     * @return
     */
    String[] getServiceUnitForServiceAssembly(String saName) throws Exception;

    /**
     * Try to shutdown all service assemblies. If a service assembly can't be
     * shut down, the shutting down process isn't aborted. It returns the list of
     * all shutdown service assemblies.
     * 
     * @return an array of successfully shutdown service assembly names.
     */
    String[] shutdownAllServiceAssemblies();

    /**
     * Try to start all service assemblies. If a service assembly can't be
     * started, the starting process isn't aborted. It returns the list of all
     * started service assemblies.
     * 
     * @return an array of successfully started service assembly names.
     */
    String[] startAllServiceAssemblies();

    /**
     * Try to stop all service assemblies. If a service assembly can't be
     * stopped, the stopping process isn't aborted. It returns the list of all
     * stopped service assemblies.
     * 
     * @return an array of successfully stopped service assembly names.
     */
    String[] stopAllServiceAssemblies();

    /**
     * Try to undeploy all service assemblies. The undeployment can be forced;
     * in this case, the deployment process is completed as this : if the SA is
     * in the STARTED state, it will be stopped and shutdown. If the SA is in
     * the STOPPED state, it will be shutdown. If a service assembly can't be
     * undeployed, the undeploying process isn't aborted. It returns the list
     * of all undeployed service assemblies.
     * 
     * @param force
     *            true if you want to force undeployment, ignoring assembly
     *            state. Default is false
     * @return an array of successfully undeployed service assembly names.
     */
    String[] undeployAllServiceAssemblies(boolean force);

}
