/**
 * PETALS - PETALS Services Platform. Copyright (c) 2007 EBM Websourcing,
 * http://www.ebmwebsourcing.com/
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version. This library is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 * -------------------------------------------------------------------------
 * $Id$
 * -------------------------------------------------------------------------
 */

package org.ow2.petals.jbi.management.task;

import java.io.File;
import java.net.URI;
import java.net.URL;

import org.ow2.petals.jbi.descriptor.original.generated.Jbi;
import org.ow2.petals.jbi.management.Context;
import org.ow2.petals.system.systemstate.SystemStateService;
import org.ow2.petals.systemstate.SystemStateException;
import org.ow2.petals.util.LoggingUtil;

/**
 * This task creates state holder entry for component, service assembly or
 * shared lib <br/>
 * The task has got a non-robust approach : it stops at the first subtask failed<br />
 * and have the undo capability.<br />
 * Inputs : {@link Jbi}, archive {@link URI}
 * 
 * @author ofabre - EBM Websourcing
 * @author fgardes - EBM Websourcing
 */
public abstract class AbstractCreateStateHolderTask extends AbstractFileManipulationTask {

    /**
     * JBI Container :: Recovery Service
     */
    protected SystemStateService systemStateService = null;

    public AbstractCreateStateHolderTask(LoggingUtil log, SystemStateService systemStateSrv,
            String petalsInstallDirectory) {
        super(log, petalsInstallDirectory);
        this.systemStateService = systemStateSrv;
    }

    public void execute(Context context) throws Exception {
        String entityName = context.getEntityName();
        String entityVersion = context.getEntityVersion();
        URL archiveUrl = context.getArchiveUrl();
        URL installUrl = context.getInstallUrl();

        createEntityStateHolder(entityName, entityVersion, installUrl, new File(archiveUrl
                .getPath()).getName());
    }

    protected abstract void createEntityStateHolder(String name, String version, URL installUrl,
            String archiveName) throws SystemStateException;

    public void undo(Context context) throws Exception {
        String entityName = context.getEntityName();
        String entityVersion = context.getEntityVersion();

        if (entityName != null) {
            try {
                this.deleteEntityStateHolder(entityName, entityVersion);
            } catch (Exception e) {
                this.log.error("Failed to revert a CreateStateHolderTask", e);
            }
        }
    }

    protected abstract void deleteEntityStateHolder(String name, String version) throws Exception;

}
