/**
 * PETALS - PETALS Services Platform. Copyright (c) 2007 EBM Websourcing,
 * http://www.ebmwebsourcing.com/
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version. This library is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 * -------------------------------------------------------------------------
 * $Id$
 * -------------------------------------------------------------------------
 */
package org.ow2.petals.jbi.management.task.installation.uninstall.sl;

import org.ow2.petals.container.ContainerService;
import org.ow2.petals.jbi.management.Context;
import org.ow2.petals.jbi.management.ManagementException;
import org.ow2.petals.jbi.management.admin.AdminService;
import org.ow2.petals.jbi.management.task.AbstractLoggableTask;
import org.ow2.petals.kernel.api.server.PetalsException;
import org.ow2.petals.util.LoggingUtil;

/**
 * This tasks removes the shared library classloader from Petals platform <br/>
 * Inputs : shared lib name<br />
 * The task has got a robust approach : it tries to do all the subtasks<br />
 * and doesn't have the undo capability.<br />
 * 
 * @author ofabre
 * @author Frederic Gardes - eBM Websourcing
 */
public class RemoveSLClassloaderTask extends AbstractLoggableTask {
    /**
     * JBI Admin Service :: JMX Agent Admin
     */
    private final AdminService adminService;

    /**
     * The Petals Container
     */
    protected ContainerService containerService;

    public RemoveSLClassloaderTask(LoggingUtil log, AdminService adminService,
            ContainerService containerService) {
        super(log);
        this.containerService = containerService;
        this.adminService = adminService;
    }

    public void execute(Context context) {
        if (context.getEntityName() != null) {
            String slName = context.getEntityName();
            String version = context.getEntityVersion();
                // Unregister
                try {
                    this.adminService.unregisterSharedLibrary(slName + "-" + version);
                } catch (ManagementException e) {
                    this.log.error("The shared library can not be unregistered", e);
                }
                
                // Remove of the fractal installer component
                try {
                    this.containerService.removeSharedLibraryLifeCycle(slName + "-" + version);
                } catch (PetalsException e) {
                    this.log.error("The fractal shared library can not be removed",
                            e);
                }
            }
    }

    public void undo(Context context) throws Exception {
        // TODO : recreate and register the shared library
    }

}
