/**
 * PETALS: PETALS Services Platform Copyright (C) 2009 EBM WebSourcing
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA.
 * 
 * Initial developer(s): EBM WebSourcing
 */
package org.ow2.petals.jbi.messaging.endpoint;

import javax.xml.namespace.QName;

import org.ow2.petals.service.ServiceEndpointImpl;
import org.ow2.petals.util.XMLUtil;
import org.w3c.dom.Document;
import org.w3c.dom.DocumentFragment;
import org.w3c.dom.Element;

import static org.ow2.petals.jbi.Constants.ENDPOINT_NAME;
import static org.ow2.petals.jbi.Constants.EPR_NAME;
import static org.ow2.petals.jbi.Constants.JBI_NAMESPACE;
import static org.ow2.petals.jbi.Constants.SERVICE_NAME;

/**
 * The JBI implementation of the petals service endpoint
 * 
 * @author chamerling - eBM WebSourcing
 * 
 */
public class JBIServiceEndpointImpl extends ServiceEndpointImpl implements ServiceEndpoint {

    /**
     * Unique serial UID
     */
    private static final long serialVersionUID = 4487649085296224233L;

    private EndpointType type;

    /**
     * 
     */
    public JBIServiceEndpointImpl() {
        super();
        this.type = EndpointType.INTERNAL;
    }

    /**
     * {@inheritDoc}
     */
    public DocumentFragment getAsReference(QName operationName) {
        Document doc = XMLUtil.newDocument();
        if (doc == null) {
            return null;
        }
        DocumentFragment result = doc.createDocumentFragment();
        Element e = doc.createElementNS(JBI_NAMESPACE, EPR_NAME);
        e.setAttribute(SERVICE_NAME, this.getServiceName().getLocalPart());
        e.setAttribute(ENDPOINT_NAME, this.getEndpointName());
        result.appendChild(e);
        return result;
    }

    /**
     * {@inheritDoc}
     */
    public QName[] getInterfaces() {
        return this.getInterfacesName().toArray(new QName[this.getInterfacesName().size()]);
    }

    @Override
    public EndpointType getType() {
        return this.type;
    }

    @Override
    public void setType(EndpointType type) {
        this.type = type;
    }
}
