/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005-2010 EBM WebSourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

package org.ow2.petals.jbi.messaging.routing.util;

import java.io.IOException;
import java.io.InputStream;
import java.util.HashMap;
import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;

import javax.jbi.messaging.MessageExchange;
import javax.jbi.messaging.MessagingException;
import javax.jbi.messaging.NormalizedMessage;
import javax.xml.transform.Source;
import javax.xml.transform.stream.StreamSource;

import org.ow2.petals.commons.stream.InputStreamForker;
import org.ow2.petals.commons.stream.ReaderInputStream;
import org.ow2.petals.jbi.messaging.exchange.MessageExchangeDecorator;

/**
 * @author rnaudin
 * 
 */
public class SourcesForkerUtil {

    /**
     * A map of forked Source Streams to set the streams reusable
     */
    private static final Map<String, Map<String, InputStreamForker>> forkedStreams = new ConcurrentHashMap<String, Map<String, InputStreamForker>>();

    /**
     * Clean the potential duplicated Sources.
     * 
     * @param exchange
     * @throws IOException
     */
    public static final void cleanExchangeSources(final MessageExchange exchange)
            throws IOException {
        final Map<String, InputStreamForker> exchangeStreamForked = forkedStreams.remove(exchange
                .getExchangeId());
        if (exchangeStreamForked != null) {
            for (final InputStreamForker streamForker : exchangeStreamForked.values()) {
                streamForker.getInputStreamTwo().close();
            }
        }
    }

    /**
     * Handle streaming issue. Duplicate sources if they are StreamSource. <br>
     * TODO : extend the mechanism to the attachments.
     * 
     * @param exchange
     * @throws MessagingException
     */
    public static final void forkExchangeSources(final MessageExchangeDecorator exchangeDecorator)
            throws MessagingException {

        Map<String, InputStreamForker> exchangeStreamForked = forkedStreams.get(exchangeDecorator
                .getExchangeId());
        if (exchangeStreamForked == null) {
            exchangeStreamForked = new HashMap<String, InputStreamForker>();
            forkedStreams.put(exchangeDecorator.getExchangeId(), exchangeStreamForked);
        }

        final Map<String, NormalizedMessage> messages = exchangeDecorator.getMessages();
        for (final String messageName : messages.keySet()) {
            // TODO Handle the Streaming issue for the Attachments??
            final Source content = messages.get(messageName).getContent();
            if (content instanceof StreamSource) {
                InputStreamForker streamForker = exchangeStreamForked.get(messageName);
                if (streamForker != null) {
                    streamForker = new InputStreamForker(streamForker.getInputStreamTwo());
                } else {
                    final StreamSource streamContent = (StreamSource) content;
                    final InputStream isContent = streamContent.getInputStream();
                    if (isContent != null) {
                        // The StreamSource was created from an InputStream
                        streamForker = new InputStreamForker(isContent);
                    } else {
                        // The StreamSource was created from a Reader
                        // we wrap it as an InputStream
                        streamForker = new InputStreamForker(new ReaderInputStream(streamContent
                                .getReader()));
                    }
                }
                exchangeStreamForked.put(messageName, streamForker);
                final Source source = new StreamSource(streamForker.getInputStreamOne());
                messages.get(messageName).setContent(source);
            }
        }
    }

}
