/**
 * PETALS: PETALS Services Platform Copyright (C) 2009 EBM WebSourcing
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA.
 * 
 * Initial developer(s): EBM WebSourcing
 */
package org.ow2.petals.ws;

import org.objectweb.fractal.api.Component;
import org.objectweb.fractal.fraclet.annotation.annotations.FractalComponent;
import org.objectweb.fractal.fraclet.annotation.annotations.Interface;
import org.objectweb.fractal.fraclet.annotation.annotations.LifeCycle;
import org.objectweb.fractal.fraclet.annotation.annotations.Monolog;
import org.objectweb.fractal.fraclet.annotation.annotations.Provides;
import org.objectweb.fractal.fraclet.annotation.annotations.Requires;
import org.objectweb.fractal.fraclet.annotation.annotations.type.LifeCycleType;
import org.objectweb.util.monolog.api.Logger;
import org.ow2.petals.container.lifecycle.ServiceAssemblyLifeCycle;
import org.ow2.petals.jbi.management.admin.AdminService;
import org.ow2.petals.kernel.ws.api.PEtALSWebServiceException;
import org.ow2.petals.kernel.ws.api.ServiceAssemblyStateService;
import org.ow2.petals.tools.ws.KernelWebService;
import org.ow2.petals.util.LoggingUtil;

/**
 * @author chamerling - eBM WebSourcing
 * 
 */
@FractalComponent
@Provides(interfaces = { @Interface(name = "service", signature = KernelWebService.class),
        @Interface(name = "webservice", signature = ServiceAssemblyStateService.class) })
public class ServiceAssemblyStateServiceImpl implements ServiceAssemblyStateService,
        KernelWebService {

    @org.objectweb.fractal.fraclet.annotation.annotations.Service(name = "component")
    private Component component;

    @Requires(name = "adminService", signature = AdminService.class)
    private AdminService adminService;

    @Monolog(name = "logger")
    private Logger logger;

    private LoggingUtil log;

    @LifeCycle(on = LifeCycleType.START)
    protected void start() {
        this.log = new LoggingUtil(this.logger);
        this.log.debug("Starting...");
    }

    @LifeCycle(on = LifeCycleType.STOP)
    protected void stop() {
        this.log.debug("Stopping...");
    }

    /**
     * {@inheritDoc}
     */
    public boolean isShutdown(String saId) throws PEtALSWebServiceException {
        ServiceAssemblyLifeCycle sa = this.getServiceAssemblyLifeCycle(saId);
        return sa.isShutdownState();
    }

    /**
     * {@inheritDoc}
     */
    public boolean isStarted(String saId) throws PEtALSWebServiceException {
        ServiceAssemblyLifeCycle sa = this.getServiceAssemblyLifeCycle(saId);
        return sa.isStartedState();
    }

    /**
     * {@inheritDoc}
     */
    public boolean isStopped(String saId) throws PEtALSWebServiceException {
        ServiceAssemblyLifeCycle sa = this.getServiceAssemblyLifeCycle(saId);
        return sa.isStoppedState();
    }

    /**
     * {@inheritDoc}
     */
    public boolean isUnknown(String saId) throws PEtALSWebServiceException {
        ServiceAssemblyLifeCycle sa = this.getServiceAssemblyLifeCycle(saId);
        return sa.isUnknownState();
    }

    /**
     * {@inheritDoc}
     */
    public Component getComponent() {
        return this.component;
    }

    private ServiceAssemblyLifeCycle getServiceAssemblyLifeCycle(final String saId)
            throws PEtALSWebServiceException {
        ServiceAssemblyLifeCycle saAssemblyLifeCycle = this.adminService
                .getServiceAssemblyByName(saId);
        if (saAssemblyLifeCycle == null) {
            throw new PEtALSWebServiceException("Can not find a service assembly with ID = " + saId);
        }
        return saAssemblyLifeCycle;
    }

}
